// chart colors
var colors = ['#007bff', '#28a745', '#333333', '#c3e6cb', '#dc3545', '#6c757d'];
$(function () {
  const Inflation = 6;
  const LifeExpectancy = 80;
  const reInvestmentReturn = 8;
  const investmentTypes = { low: 7, medium: 8.5, high: 10 };
  const monthlyExpText = "Monthly Expenses projected at Retirement age of [retiredage] will be";
  var canvas = document.getElementById("chLine");
  var ctx = canvas.getContext('2d');
  ctx.textAlign = "center";
  ctx.font = "13px Arial";
  ctx.fillText("Calculate to view corpus graph.", canvas.width / 2, canvas.height / 2);

  $('#detailReport').hide();
  $('#loadContent').hide();

  var canvasBar = document.getElementById("chBar");
  var ctxBar = canvasBar.getContext('2d');
  ctxBar.textAlign = "center";
  ctxBar.font = "13px Arial";
  ctxBar.fillText("Calculate to view corpus graph.", canvasBar.width / 2, canvasBar.height / 2);
  var myBarChart = null;
  var myLineChart = null;

  $("#investmentForm").on("submit", function (event) {
    event.preventDefault();
    $('#loadContent').show();
    let age = $("#age").val();
    let monthlyExp = convertNumber($("#monthlyExp").val());
    let retireIn = $("#retireIn").val();
    if (retireIn > LifeExpectancy) {
      alert("Retirement is not greater than life expectancy");
      return;
    }
    let savedAmt = convertNumber($("#savedAmt").val());
    var investmentType = $("input[name='investmentType']:checked").val();
    let ContributionsExp = convertNumber($("#ContributionsExp").val());
    let retiredAge = parseInt(age) + parseInt(retireIn);
    let reMonthlyExp = getFutureValue(Inflation / 100, retireIn, 0, -monthlyExp, 1);
    let rateduration = (1 + reInvestmentReturn / 100) / (1 + Inflation / 100) - 1;
    let nper = LifeExpectancy - (retiredAge);
    let reCorpus = getPresentvalue(rateduration, nper, -reMonthlyExp * 12, 0, 1);
    let nominalValue = getNominal(investmentTypes[investmentType] / 100, 12) / 12;
    let requiredMonthlyContribution = getPMT(nominalValue, retireIn * 12, savedAmt, -reCorpus, 1);
    let corpusPresent = getFutureValue(nominalValue, retireIn * 12, -ContributionsExp, -savedAmt, 1);

    if (myBarChart == null)
      myBarChart = createBarChart([rountup(corpusPresent), rountup(reCorpus)]);
    else
      updateBarchat(myBarChart, [rountup(corpusPresent), rountup(reCorpus)]);
    let labels = [retiredAge];
    let expanses = [rountup(reMonthlyExp * 12)];
    let corpus = [rountup(corpusPresent)];
    let index = 0;
    for (let i = retiredAge; i < LifeExpectancy; i++) {
      labels.push(i + 1);
      expanse = expanses[index] * (1 + (Inflation / 100));
      corpu = (corpus[index] * (1 + (reInvestmentReturn / 100))) - expanses[index];
      expanses.push(rountup(expanse));
      corpus.push(rountup(corpu));
      if (corpu < 0)
        break;
      index += 1;
    }
    console.log("corpus", corpus, expanses, labels);
    if (myLineChart == null)
      myLineChart = createLineChart(corpus, expanses, labels);
    else
      updateLineChart(myLineChart, corpus, expanses, labels);
    $('#reMonthlyExpText').html(monthlyExpText.replace("[retiredage]", retiredAge));
    $('#reMonthlyExp').html("₹ " + currenyFormat(rountup(reMonthlyExp)));
    $('#reCorpus').text("₹ " + currenyFormat(rountup(reCorpus)));
    $('#requiredMonthlyContribution').text("₹ " + currenyFormat(rountup(requiredMonthlyContribution)));
    $('#detailReport').show();
    setTimeout(() => {
      $('#loadContent').hide();
    }, 1000);
  });

});

function convertNumber(val) {
  return parseInt(val ? val.replaceAll(',', '') : 0);
}

function currenyFormat(value) {
  if (value < 99999) {
    return Intl.NumberFormat('en-IN').format(value);
  }
  return extention(Intl.NumberFormat('en-IN', {
    notation: "compact",
    compactDisplay: "short",
  }).format(value));
  // .replace("LCr", " lakh Crore").replace("TCr", " Thousand Crore").replace("Cr", " Crore").replace("L", " lakh").replace("T", " Thousand");
}

function extention(value) {
  if (value.includes("LCr")) {
    value = value.replace("LCr", " lakh Crore");
  } else if (value.includes("TCr")) {
    value = value.replace("TCr", " Thousand Crore");
  } else if (value.includes("Cr")) {
    value = value.replace("Cr", " Crore");
  } else if (value.includes("L")) {
    value = value.replace("L", " lakh");
  } else if (value.includes("T")) {
    value = value.replace("T", " Thousand");
  }
  return value;
}

function rountup(x) {
  return Math.round(x);
}

// Parameters are rate, total number of periods, payment made each period and future value
function getPresentvalue(rate, nper, pmt, fv, type) {
  rate = parseFloat(rate);
  nper = parseFloat(nper);
  pmt = parseFloat(pmt);
  fv = parseFloat(fv);
  if (nper == 0) {
    return (0);
  }
  if (rate == 0) { // Interest rate is 0
    pv_value = -(fv + (pmt * nper));
  } else {
    y = Math.pow(1 + rate, nper);
    pv_value = (((1 - y) / rate) * pmt * (1 + rate * type) - fv) / y;
  }
  return (pv_value);
}

function getFutureValue(rate, nper, pmt, pv, type) {
  rate = parseFloat(rate);
  nper = parseFloat(nper);
  pmt = parseFloat(pmt);
  pv = parseFloat(pv);
  if (nper == 0) {
    return (0);
  }
  if (rate == 0) { // Interest rate is 0
    fv_value = -(pv + (pmt * nper));
  } else {
    x = Math.pow(1 + rate, nper);
    fv_value = (pmt * (1 + rate * type) * (1 - x) / rate) - pv * x;
  }
  return (fv_value);
}

function getPMT(rate, nper, pv, fv, type) {
  if (!fv) fv = 0;
  if (!type) type = 0;

  if (rate == 0) return -(pv + fv) / nper;

  var pvif = Math.pow(1 + rate, nper);
  var pmt = rate / (pvif - 1) * -(pv * pvif + fv);

  if (type == 1) {
    pmt /= (1 + rate);
  };
  return pmt;
}

function getNominal(rate, nper) {
  let nRate = nper * (Math.pow(1 + rate, 1 / nper) - 1)
  return nRate;
}

function NPER(rate, payment, present, future, type) {
  // Initialize type
  type = (typeof type === 'undefined') ? 0 : type;

  // Initialize future value
  future = (typeof future === 'undefined') ? 0 : future;

  // Return number of periods
  const num = payment * (1 + rate * type) - future * rate;
  const den = (present * rate + payment * (1 + rate * type));
  return Math.log(num / den) / Math.log(1 + rate);
}

function updateBarchat(mychart, data) {
  mychart.data.datasets[0].data = data;
  mychart.update();
}

function createBarChart(data) {
  /* bar chart */
  var chBar = document.getElementById("chBar");
  var myBarchartx = null;
  if (chBar) {
    myBarchartx = new Chart(chBar, {
      type: 'bar',
      data: {
        labels: ["What you will have", "What you will need"],
        datasets: [{
          data: data,
          backgroundColor: [colors[0], colors[1]]
        }]
      },


      options: {
        legend: {
          display: false
        },

        scales: {
          xAxes: [{
            barPercentage: 0.4,
            categoryPercentage: 0.5
          }],
          yAxes: [{
            ticks: {
              beginAtZero: true,
              callback: function (value, index, values) {
                return Intl.NumberFormat('en-IN', {
                  notation: "compact",
                  compactDisplay: "short",
                }).format(value);
              }
            }
          }]
        }
      }
    });
  }
  return myBarchartx;
}

function createLineChart(corpus, expenses, labels) {
  /* large line chart */
  var myLineChartx = null;
  var chLine = document.getElementById("chLine");
  var chartData = {
    labels: labels,
    datasets: [{
      label: "Corpus",
      data: corpus,
      backgroundColor: 'transparent',
      borderColor: colors[0],
      borderWidth: 4,
      pointBackgroundColor: colors[0]
    },

    {
      label: "Expenses",
      data: expenses,
      backgroundColor: colors[3],
      borderColor: colors[1],
      borderWidth: 4,
      pointBackgroundColor: colors[1]
    }
    ]
  };

  if (chLine) {
    myLineChartx = new Chart(chLine, {
      type: 'line',
      data: chartData,
      options: {
        scales: {
          xAxes: [{
            scaleLabel: {
              display: true,
              labelString: 'Age'
            },
            ticks: {
              beginAtZero: false
            }
          }],
          yAxes: [{
            ticks: {
              beginAtZero: false,
              callback: function (value, index, values) {
                return Intl.NumberFormat('en-IN', {
                  notation: "compact",
                  compactDisplay: "short",
                }).format(value);
              }
            }
          }]
        },

        legend: {
          display: true,
          position: "bottom",
        },

        responsive: true
      }
    });
  }
  return myLineChartx;
}

function updateLineChart(mychart, corpus, expenses, labels) {
  mychart.data.labels = labels;
  mychart.data.datasets[0].data = corpus;
  mychart.data.datasets[1].data = expenses;
  mychart.update();
}

// Jquery Dependency

$("input[data-type='currency']").on({
  keyup: function () {
    formatCurrency($(this));
  }
});

function formatNumber(n) {
  // format number 1000000 to 1,234,567
  return n.replace(/\D/g, "").replace(/\B(?=(\d{3})+(?!\d))/g, ",")
}


function formatCurrency(input) {
  // skip for arrow keys
  if (input.which >= 37 && input.which <= 40) return;

  // format number
  input.val(function (index, value) {
    return value.replace(/\D/g, "").replace(/\B(?=(\d{3})+(?!\d))/g, ",");
  });
}

function isNumber(evt) {
  evt = (evt) ? evt : window.event;
  var charCode = (evt.which) ? evt.which : evt.keyCode;
  if (charCode > 31 && (charCode < 48 || charCode > 57)) {
    return false;
  }
  return true;
}

//// slider /////

// const slider = document.getElementById('ContributionsExp');
// const sliderValue = document.getElementById('input-slider-red');
// slider.addEventListener('input', () => {
//   const value = Number(slider.value);
//   const valueWithCommas = value.toLocaleString();

//   sliderValue.value = valueWithCommas;
// });

// sliderValue.addEventListener('input', () => {
//   const valueWithoutCommas = sliderValue.value.replace(/,/g, '');
//   const parsedValue = parseInt(valueWithoutCommas);

//   if (!isNaN(parsedValue)) {
//     const valueWithCommas = parsedValue.toLocaleString();
//     sliderValue.value = valueWithCommas;
//     slider.value = parsedValue;
//   }
// });


// const slider1 = document.getElementById('savedAmt');
// const sliderValue1 = document.getElementById('input-slider-green');
// slider1.addEventListener('input', () => {
//   const value = Number(slider1.value);
//   const valueWithCommas = value.toLocaleString();

//   sliderValue1.value = valueWithCommas;
// });

// sliderValue1.addEventListener('input', () => {
//   const valueWithoutCommas = sliderValue1.value.replace(/,/g, '');
//   const parsedValue = parseInt(valueWithoutCommas);

//   if (!isNaN(parsedValue)) {
//     const valueWithCommas = parsedValue.toLocaleString();
//     sliderValue1.value = valueWithCommas;
//     slider1.value = parsedValue;
//   }
// });


// const slider2 = document.getElementById('monthlyExp');
// const sliderValue2 = document.getElementById('input-slider-blue');
// slider2.addEventListener('input', () => {
//   const value = Number(slider2.value);
//   const valueWithCommas = value.toLocaleString();

//   sliderValue2.value = valueWithCommas;
// });

// sliderValue2.addEventListener('input', () => {
//   const valueWithoutCommas = sliderValue2.value.replace(/,/g, '');
//   const parsedValue = parseInt(valueWithoutCommas);

//   if (!isNaN(parsedValue)) {
//     const valueWithCommas = parsedValue.toLocaleString();
//     sliderValue2.value = valueWithCommas;
//     slider2.value = parsedValue;
//   }
// });



const slider2 = document.getElementById('monthlyExp');
const sliderValue2 = document.getElementById('input-slider-blue');

function formatCurrency(number) {
  const formatter = new Intl.NumberFormat('en-IN');
  return formatter.format(number);
}

slider2.addEventListener('input', () => {
  const value = Number(slider2.value);
  const valueFormatted = formatCurrency(value);

  sliderValue2.value = valueFormatted;
});

sliderValue2.addEventListener('input', () => {
  const valueWithoutCommas = sliderValue2.value.replace(/,/g, '');
  const parsedValue = parseInt(valueWithoutCommas);

  if (!isNaN(parsedValue)) {
    const valueFormatted = formatCurrency(parsedValue);
    sliderValue2.value = valueFormatted;
    slider2.value = parsedValue;
  }
});

////
const slider = document.getElementById('savedAmt');
const sliderValue = document.getElementById('input-slider-green');

function formatCurrency(number) {
  const formatter = new Intl.NumberFormat('en-IN');
  return formatter.format(number);
}

slider.addEventListener('input', () => {
  const value = Number(slider.value);
  const valueFormatted = formatCurrency(value);

  sliderValue.value = valueFormatted;
});

sliderValue.addEventListener('input', () => {
  const valueWithoutCommas = sliderValue.value.replace(/,/g, '');
  const parsedValue = parseInt(valueWithoutCommas);

  if (!isNaN(parsedValue)) {
    const valueFormatted = formatCurrency(parsedValue);
    sliderValue.value = valueFormatted;
    slider.value = parsedValue;
  }
});
/////
const slider1 = document.getElementById('ContributionsExp');
const sliderValue1 = document.getElementById('input-slider-red');

function formatCurrency(number) {
  const formatter = new Intl.NumberFormat('en-IN');
  return formatter.format(number);
}

slider1.addEventListener('input', () => {
  const value = Number(slider1.value);
  const valueFormatted = formatCurrency(value);

  sliderValue1.value = valueFormatted;
});

sliderValue1.addEventListener('input', () => {
  const valueWithoutCommas = sliderValue1.value.replace(/,/g, '');
  const parsedValue = parseInt(valueWithoutCommas);

  if (!isNaN(parsedValue)) {
    const valueFormatted = formatCurrency(parsedValue);
    sliderValue1.value = valueFormatted;
    slider1.value = parsedValue;
  }
});
